/*
    -- MAGMA (version 2.5.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date January 2019

       @generated from include/magmablas_zc_v1_map.h, mixed zc -> ds, Wed Jan  2 14:18:55 2019
*/

#ifndef MAGMABLAS_DS_V1_MAP_H
#define MAGMABLAS_DS_V1_MAP_H

// =============================================================================
// map function names to old v1 routines

#define magmablas_dsaxpycp                  magmablas_dsaxpycp_v1
#define magmablas_dslaswp                   magmablas_dslaswp_v1
#define magmablas_dlag2s                    magmablas_dlag2s_v1
#define magmablas_slag2d                    magmablas_slag2d_v1
#define magmablas_dlat2s                    magmablas_dlat2s_v1
#define magmablas_slat2d                    magmablas_slat2d_v1

#endif // MAGMABLAS_DS_V1_MAP_H
