/*
    -- MAGMA (version 2.5.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date January 2019
*/

#ifndef MAGMABLAS_H
#define MAGMABLAS_H

#include "magma_copy.h"
#include "magmablas_z.h"
#include "magmablas_c.h"
#include "magmablas_d.h"
#include "magmablas_s.h"
#include "magmablas_zc.h"
#include "magmablas_ds.h"
#include "magmablas_h.h"

#endif // MAGMABLAS_H
