/*
    -- MAGMA (version 2.5.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date January 2019

       @precisions mixed zc -> ds
*/

#ifndef MAGMABLAS_ZC_V1_MAP_H
#define MAGMABLAS_ZC_V1_MAP_H

// =============================================================================
// map function names to old v1 routines

#define magmablas_zcaxpycp                  magmablas_zcaxpycp_v1
#define magmablas_zclaswp                   magmablas_zclaswp_v1
#define magmablas_zlag2c                    magmablas_zlag2c_v1
#define magmablas_clag2z                    magmablas_clag2z_v1
#define magmablas_zlat2c                    magmablas_zlat2c_v1
#define magmablas_clat2z                    magmablas_clat2z_v1

#endif // MAGMABLAS_ZC_V1_MAP_H
